@@ -14,7 +14,7 @@ NTSTATUS dcerpc_BrowserrQueryOtherDomains_r(struct dcerpc_binding_handle *h, TAL
 struct tevent_req *dcerpc_BrowserrQueryOtherDomains_send(TALLOC_CTX *mem_ctx,
 							 struct tevent_context *ev,
 							 struct dcerpc_binding_handle *h,
-							 const char *_server_unc /* [in] [unique,charset(UTF16)] */,
+							 const char *_server_unc /* [in] [charset(UTF16),unique] */,
 							 struct BrowserrSrvInfo *_info /* [in,out] [ref] */,
 							 uint32_t *_total_entries /* [out] [ref] */);
 NTSTATUS dcerpc_BrowserrQueryOtherDomains_recv(struct tevent_req *req,
@@ -22,7 +22,7 @@ NTSTATUS dcerpc_BrowserrQueryOtherDomains_recv(struct tevent_req *req,
 					       WERROR *result);
 NTSTATUS dcerpc_BrowserrQueryOtherDomains(struct dcerpc_binding_handle *h,
 					  TALLOC_CTX *mem_ctx,
-					  const char *_server_unc /* [in] [unique,charset(UTF16)] */,
+					  const char *_server_unc /* [in] [charset(UTF16),unique] */,
 					  struct BrowserrSrvInfo *_info /* [in,out] [ref] */,
 					  uint32_t *_total_entries /* [out] [ref] */,
 					  WERROR *result);
