@@ -78,9 +78,9 @@ struct tevent_req *dcerpc_drsuapi_DsGetNCChanges_send(TALLOC_CTX *mem_ctx,
 						      struct dcerpc_binding_handle *h,
 						      struct policy_handle *_bind_handle /* [in] [ref] */,
 						      uint32_t _level /* [in]  */,
-						      union drsuapi_DsGetNCChangesRequest *_req /* [in] [ref,switch_is(level)] */,
+						      union drsuapi_DsGetNCChangesRequest *_req /* [in] [switch_is(level),ref] */,
 						      uint32_t *_level_out /* [out] [ref] */,
-						      union drsuapi_DsGetNCChangesCtr *_ctr /* [out] [ref,switch_is(*level_out)] */);
+						      union drsuapi_DsGetNCChangesCtr *_ctr /* [out] [switch_is(*level_out),ref] */);
 NTSTATUS dcerpc_drsuapi_DsGetNCChanges_recv(struct tevent_req *req,
 					    TALLOC_CTX *mem_ctx,
 					    WERROR *result);
@@ -88,9 +88,9 @@ NTSTATUS dcerpc_drsuapi_DsGetNCChanges(struct dcerpc_binding_handle *h,
 				       TALLOC_CTX *mem_ctx,
 				       struct policy_handle *_bind_handle /* [in] [ref] */,
 				       uint32_t _level /* [in]  */,
-				       union drsuapi_DsGetNCChangesRequest *_req /* [in] [ref,switch_is(level)] */,
+				       union drsuapi_DsGetNCChangesRequest *_req /* [in] [switch_is(level),ref] */,
 				       uint32_t *_level_out /* [out] [ref] */,
-				       union drsuapi_DsGetNCChangesCtr *_ctr /* [out] [ref,switch_is(*level_out)] */,
+				       union drsuapi_DsGetNCChangesCtr *_ctr /* [out] [switch_is(*level_out),ref] */,
 				       WERROR *result);
 
 struct tevent_req *dcerpc_drsuapi_DsReplicaUpdateRefs_r_send(TALLOC_CTX *mem_ctx,
@@ -194,7 +194,7 @@ struct tevent_req *dcerpc_drsuapi_DsGetMemberships_send(TALLOC_CTX *mem_ctx,
 							uint32_t _level /* [in]  */,
 							union drsuapi_DsGetMembershipsRequest *_req /* [in] [ref,switch_is(level)] */,
 							uint32_t *_level_out /* [out] [ref] */,
-							union drsuapi_DsGetMembershipsCtr *_ctr /* [out] [ref,switch_is(*level_out)] */);
+							union drsuapi_DsGetMembershipsCtr *_ctr /* [out] [switch_is(*level_out),ref] */);
 NTSTATUS dcerpc_drsuapi_DsGetMemberships_recv(struct tevent_req *req,
 					      TALLOC_CTX *mem_ctx,
 					      WERROR *result);
@@ -204,7 +204,7 @@ NTSTATUS dcerpc_drsuapi_DsGetMemberships(struct dcerpc_binding_handle *h,
 					 uint32_t _level /* [in]  */,
 					 union drsuapi_DsGetMembershipsRequest *_req /* [in] [ref,switch_is(level)] */,
 					 uint32_t *_level_out /* [out] [ref] */,
-					 union drsuapi_DsGetMembershipsCtr *_ctr /* [out] [ref,switch_is(*level_out)] */,
+					 union drsuapi_DsGetMembershipsCtr *_ctr /* [out] [switch_is(*level_out),ref] */,
 					 WERROR *result);
 
 struct tevent_req *dcerpc_drsuapi_DsGetNT4ChangeLog_r_send(TALLOC_CTX *mem_ctx,
@@ -218,7 +218,7 @@ struct tevent_req *dcerpc_drsuapi_DsGetNT4ChangeLog_send(TALLOC_CTX *mem_ctx,
 							 struct dcerpc_binding_handle *h,
 							 struct policy_handle *_bind_handle /* [in] [ref] */,
 							 uint32_t _level /* [in]  */,
-							 union drsuapi_DsGetNT4ChangeLogRequest *_req /* [in] [ref,switch_is(level)] */,
+							 union drsuapi_DsGetNT4ChangeLogRequest *_req /* [in] [switch_is(level),ref] */,
 							 uint32_t *_level_out /* [out] [ref] */,
 							 union drsuapi_DsGetNT4ChangeLogInfo *_info /* [out] [ref,switch_is(*level_out)] */);
 NTSTATUS dcerpc_drsuapi_DsGetNT4ChangeLog_recv(struct tevent_req *req,
@@ -228,7 +228,7 @@ NTSTATUS dcerpc_drsuapi_DsGetNT4ChangeLog(struct dcerpc_binding_handle *h,
 					  TALLOC_CTX *mem_ctx,
 					  struct policy_handle *_bind_handle /* [in] [ref] */,
 					  uint32_t _level /* [in]  */,
-					  union drsuapi_DsGetNT4ChangeLogRequest *_req /* [in] [ref,switch_is(level)] */,
+					  union drsuapi_DsGetNT4ChangeLogRequest *_req /* [in] [switch_is(level),ref] */,
 					  uint32_t *_level_out /* [out] [ref] */,
 					  union drsuapi_DsGetNT4ChangeLogInfo *_info /* [out] [ref,switch_is(*level_out)] */,
 					  WERROR *result);
@@ -246,7 +246,7 @@ struct tevent_req *dcerpc_drsuapi_DsCrackNames_send(TALLOC_CTX *mem_ctx,
 						    uint32_t _level /* [in]  */,
 						    union drsuapi_DsNameRequest *_req /* [in] [ref,switch_is(level)] */,
 						    uint32_t *_level_out /* [out] [ref] */,
-						    union drsuapi_DsNameCtr *_ctr /* [out] [ref,switch_is(*level_out)] */);
+						    union drsuapi_DsNameCtr *_ctr /* [out] [switch_is(*level_out),ref] */);
 NTSTATUS dcerpc_drsuapi_DsCrackNames_recv(struct tevent_req *req,
 					  TALLOC_CTX *mem_ctx,
 					  WERROR *result);
@@ -256,7 +256,7 @@ NTSTATUS dcerpc_drsuapi_DsCrackNames(struct dcerpc_binding_handle *h,
 				     uint32_t _level /* [in]  */,
 				     union drsuapi_DsNameRequest *_req /* [in] [ref,switch_is(level)] */,
 				     uint32_t *_level_out /* [out] [ref] */,
-				     union drsuapi_DsNameCtr *_ctr /* [out] [ref,switch_is(*level_out)] */,
+				     union drsuapi_DsNameCtr *_ctr /* [out] [switch_is(*level_out),ref] */,
 				     WERROR *result);
 
 struct tevent_req *dcerpc_drsuapi_DsWriteAccountSpn_r_send(TALLOC_CTX *mem_ctx,
@@ -272,7 +272,7 @@ struct tevent_req *dcerpc_drsuapi_DsWriteAccountSpn_send(TALLOC_CTX *mem_ctx,
 							 uint32_t _level /* [in]  */,
 							 union drsuapi_DsWriteAccountSpnRequest *_req /* [in] [ref,switch_is(level)] */,
 							 uint32_t *_level_out /* [out] [ref] */,
-							 union drsuapi_DsWriteAccountSpnResult *_res /* [out] [ref,switch_is(*level_out)] */);
+							 union drsuapi_DsWriteAccountSpnResult *_res /* [out] [switch_is(*level_out),ref] */);
 NTSTATUS dcerpc_drsuapi_DsWriteAccountSpn_recv(struct tevent_req *req,
 					       TALLOC_CTX *mem_ctx,
 					       WERROR *result);
@@ -282,7 +282,7 @@ NTSTATUS dcerpc_drsuapi_DsWriteAccountSpn(struct dcerpc_binding_handle *h,
 					  uint32_t _level /* [in]  */,
 					  union drsuapi_DsWriteAccountSpnRequest *_req /* [in] [ref,switch_is(level)] */,
 					  uint32_t *_level_out /* [out] [ref] */,
-					  union drsuapi_DsWriteAccountSpnResult *_res /* [out] [ref,switch_is(*level_out)] */,
+					  union drsuapi_DsWriteAccountSpnResult *_res /* [out] [switch_is(*level_out),ref] */,
 					  WERROR *result);
 
 struct tevent_req *dcerpc_drsuapi_DsRemoveDSServer_r_send(TALLOC_CTX *mem_ctx,
@@ -322,7 +322,7 @@ struct tevent_req *dcerpc_drsuapi_DsGetDomainControllerInfo_send(TALLOC_CTX *mem
 								 struct dcerpc_binding_handle *h,
 								 struct policy_handle *_bind_handle /* [in] [ref] */,
 								 int32_t _level /* [in]  */,
-								 union drsuapi_DsGetDCInfoRequest *_req /* [in] [ref,switch_is(level)] */,
+								 union drsuapi_DsGetDCInfoRequest *_req /* [in] [switch_is(level),ref] */,
 								 int32_t *_level_out /* [out] [ref] */,
 								 union drsuapi_DsGetDCInfoCtr *_ctr /* [out] [ref,switch_is(*level_out)] */);
 NTSTATUS dcerpc_drsuapi_DsGetDomainControllerInfo_recv(struct tevent_req *req,
@@ -332,7 +332,7 @@ NTSTATUS dcerpc_drsuapi_DsGetDomainControllerInfo(struct dcerpc_binding_handle *
 						  TALLOC_CTX *mem_ctx,
 						  struct policy_handle *_bind_handle /* [in] [ref] */,
 						  int32_t _level /* [in]  */,
-						  union drsuapi_DsGetDCInfoRequest *_req /* [in] [ref,switch_is(level)] */,
+						  union drsuapi_DsGetDCInfoRequest *_req /* [in] [switch_is(level),ref] */,
 						  int32_t *_level_out /* [out] [ref] */,
 						  union drsuapi_DsGetDCInfoCtr *_ctr /* [out] [ref,switch_is(*level_out)] */,
 						  WERROR *result);
@@ -348,7 +348,7 @@ struct tevent_req *dcerpc_drsuapi_DsAddEntry_send(TALLOC_CTX *mem_ctx,
 						  struct dcerpc_binding_handle *h,
 						  struct policy_handle *_bind_handle /* [in] [ref] */,
 						  uint32_t _level /* [in]  */,
-						  union drsuapi_DsAddEntryRequest *_req /* [in] [ref,switch_is(level)] */,
+						  union drsuapi_DsAddEntryRequest *_req /* [in] [switch_is(level),ref] */,
 						  uint32_t *_level_out /* [out] [ref] */,
 						  union drsuapi_DsAddEntryCtr *_ctr /* [out] [ref,switch_is(*level_out)] */);
 NTSTATUS dcerpc_drsuapi_DsAddEntry_recv(struct tevent_req *req,
@@ -358,7 +358,7 @@ NTSTATUS dcerpc_drsuapi_DsAddEntry(struct dcerpc_binding_handle *h,
 				   TALLOC_CTX *mem_ctx,
 				   struct policy_handle *_bind_handle /* [in] [ref] */,
 				   uint32_t _level /* [in]  */,
-				   union drsuapi_DsAddEntryRequest *_req /* [in] [ref,switch_is(level)] */,
+				   union drsuapi_DsAddEntryRequest *_req /* [in] [switch_is(level),ref] */,
 				   uint32_t *_level_out /* [out] [ref] */,
 				   union drsuapi_DsAddEntryCtr *_ctr /* [out] [ref,switch_is(*level_out)] */,
 				   WERROR *result);
@@ -396,9 +396,9 @@ struct tevent_req *dcerpc_drsuapi_DsReplicaGetInfo_send(TALLOC_CTX *mem_ctx,
 							struct dcerpc_binding_handle *h,
 							struct policy_handle *_bind_handle /* [in] [ref] */,
 							enum drsuapi_DsReplicaGetInfoLevel _level /* [in]  */,
-							union drsuapi_DsReplicaGetInfoRequest *_req /* [in] [ref,switch_is(level)] */,
+							union drsuapi_DsReplicaGetInfoRequest *_req /* [in] [switch_is(level),ref] */,
 							enum drsuapi_DsReplicaInfoType *_info_type /* [out] [ref] */,
-							union drsuapi_DsReplicaInfo *_info /* [out] [ref,switch_is(*info_type)] */);
+							union drsuapi_DsReplicaInfo *_info /* [out] [switch_is(*info_type),ref] */);
 NTSTATUS dcerpc_drsuapi_DsReplicaGetInfo_recv(struct tevent_req *req,
 					      TALLOC_CTX *mem_ctx,
 					      WERROR *result);
@@ -406,9 +406,9 @@ NTSTATUS dcerpc_drsuapi_DsReplicaGetInfo(struct dcerpc_binding_handle *h,
 					 TALLOC_CTX *mem_ctx,
 					 struct policy_handle *_bind_handle /* [in] [ref] */,
 					 enum drsuapi_DsReplicaGetInfoLevel _level /* [in]  */,
-					 union drsuapi_DsReplicaGetInfoRequest *_req /* [in] [ref,switch_is(level)] */,
+					 union drsuapi_DsReplicaGetInfoRequest *_req /* [in] [switch_is(level),ref] */,
 					 enum drsuapi_DsReplicaInfoType *_info_type /* [out] [ref] */,
-					 union drsuapi_DsReplicaInfo *_info /* [out] [ref,switch_is(*info_type)] */,
+					 union drsuapi_DsReplicaInfo *_info /* [out] [switch_is(*info_type),ref] */,
 					 WERROR *result);
 
 struct tevent_req *dcerpc_drsuapi_DsGetMemberships2_r_send(TALLOC_CTX *mem_ctx,
@@ -450,7 +450,7 @@ struct tevent_req *dcerpc_drsuapi_QuerySitesByCost_send(TALLOC_CTX *mem_ctx,
 							uint32_t _level /* [in]  */,
 							union drsuapi_QuerySitesByCostRequest *_req /* [in] [ref,switch_is(level)] */,
 							uint32_t *_level_out /* [out] [ref] */,
-							union drsuapi_QuerySitesByCostCtr *_ctr /* [out] [ref,switch_is(*level_out)] */);
+							union drsuapi_QuerySitesByCostCtr *_ctr /* [out] [switch_is(*level_out),ref] */);
 NTSTATUS dcerpc_drsuapi_QuerySitesByCost_recv(struct tevent_req *req,
 					      TALLOC_CTX *mem_ctx,
 					      WERROR *result);
@@ -460,7 +460,7 @@ NTSTATUS dcerpc_drsuapi_QuerySitesByCost(struct dcerpc_binding_handle *h,
 					 uint32_t _level /* [in]  */,
 					 union drsuapi_QuerySitesByCostRequest *_req /* [in] [ref,switch_is(level)] */,
 					 uint32_t *_level_out /* [out] [ref] */,
-					 union drsuapi_QuerySitesByCostCtr *_ctr /* [out] [ref,switch_is(*level_out)] */,
+					 union drsuapi_QuerySitesByCostCtr *_ctr /* [out] [switch_is(*level_out),ref] */,
 					 WERROR *result);
 
 #endif /* _HEADER_RPC_drsuapi */
