@@ -166,7 +166,7 @@ struct tevent_req *dcerpc_lsa_SetInfoPolicy_send(TALLOC_CTX *mem_ctx,
 						 struct dcerpc_binding_handle *h,
 						 struct policy_handle *_handle /* [in] [ref] */,
 						 enum lsa_PolicyInfo _level /* [in]  */,
-						 union lsa_PolicyInformation *_info /* [in] [ref,switch_is(level)] */);
+						 union lsa_PolicyInformation *_info /* [in] [switch_is(level),ref] */);
 NTSTATUS dcerpc_lsa_SetInfoPolicy_recv(struct tevent_req *req,
 				       TALLOC_CTX *mem_ctx,
 				       NTSTATUS *result);
@@ -174,7 +174,7 @@ NTSTATUS dcerpc_lsa_SetInfoPolicy(struct dcerpc_binding_handle *h,
 				  TALLOC_CTX *mem_ctx,
 				  struct policy_handle *_handle /* [in] [ref] */,
 				  enum lsa_PolicyInfo _level /* [in]  */,
-				  union lsa_PolicyInformation *_info /* [in] [ref,switch_is(level)] */,
+				  union lsa_PolicyInformation *_info /* [in] [switch_is(level),ref] */,
 				  NTSTATUS *result);
 
 struct tevent_req *dcerpc_lsa_CreateAccount_r_send(TALLOC_CTX *mem_ctx,
@@ -516,7 +516,7 @@ struct tevent_req *dcerpc_lsa_QueryTrustedDomainInfo_send(TALLOC_CTX *mem_ctx,
 							  struct dcerpc_binding_handle *h,
 							  struct policy_handle *_trustdom_handle /* [in] [ref] */,
 							  enum lsa_TrustDomInfoEnum _level /* [in]  */,
-							  union lsa_TrustedDomainInfo **_info /* [out] [ref,switch_is(level)] */);
+							  union lsa_TrustedDomainInfo **_info /* [out] [switch_is(level),ref] */);
 NTSTATUS dcerpc_lsa_QueryTrustedDomainInfo_recv(struct tevent_req *req,
 						TALLOC_CTX *mem_ctx,
 						NTSTATUS *result);
@@ -524,7 +524,7 @@ NTSTATUS dcerpc_lsa_QueryTrustedDomainInfo(struct dcerpc_binding_handle *h,
 					   TALLOC_CTX *mem_ctx,
 					   struct policy_handle *_trustdom_handle /* [in] [ref] */,
 					   enum lsa_TrustDomInfoEnum _level /* [in]  */,
-					   union lsa_TrustedDomainInfo **_info /* [out] [ref,switch_is(level)] */,
+					   union lsa_TrustedDomainInfo **_info /* [out] [switch_is(level),ref] */,
 					   NTSTATUS *result);
 
 struct tevent_req *dcerpc_lsa_SetInformationTrustedDomain_r_send(TALLOC_CTX *mem_ctx,
@@ -922,7 +922,7 @@ NTSTATUS dcerpc_lsa_OpenPolicy2_r(struct dcerpc_binding_handle *h, TALLOC_CTX *m
 struct tevent_req *dcerpc_lsa_OpenPolicy2_send(TALLOC_CTX *mem_ctx,
 					       struct tevent_context *ev,
 					       struct dcerpc_binding_handle *h,
-					       const char *_system_name /* [in] [unique,charset(UTF16)] */,
+					       const char *_system_name /* [in] [charset(UTF16),unique] */,
 					       struct lsa_ObjectAttribute *_attr /* [in] [ref] */,
 					       uint32_t _access_mask /* [in]  */,
 					       struct policy_handle *_handle /* [out] [ref] */);
@@ -931,7 +931,7 @@ NTSTATUS dcerpc_lsa_OpenPolicy2_recv(struct tevent_req *req,
 				     NTSTATUS *result);
 NTSTATUS dcerpc_lsa_OpenPolicy2(struct dcerpc_binding_handle *h,
 				TALLOC_CTX *mem_ctx,
-				const char *_system_name /* [in] [unique,charset(UTF16)] */,
+				const char *_system_name /* [in] [charset(UTF16),unique] */,
 				struct lsa_ObjectAttribute *_attr /* [in] [ref] */,
 				uint32_t _access_mask /* [in]  */,
 				struct policy_handle *_handle /* [out] [ref] */,
@@ -946,7 +946,7 @@ NTSTATUS dcerpc_lsa_GetUserName_r(struct dcerpc_binding_handle *h, TALLOC_CTX *m
 struct tevent_req *dcerpc_lsa_GetUserName_send(TALLOC_CTX *mem_ctx,
 					       struct tevent_context *ev,
 					       struct dcerpc_binding_handle *h,
-					       const char *_system_name /* [in] [unique,charset(UTF16)] */,
+					       const char *_system_name /* [in] [charset(UTF16),unique] */,
 					       struct lsa_String **_account_name /* [in,out] [ref] */,
 					       struct lsa_String **_authority_name /* [in,out] [unique] */);
 NTSTATUS dcerpc_lsa_GetUserName_recv(struct tevent_req *req,
@@ -954,7 +954,7 @@ NTSTATUS dcerpc_lsa_GetUserName_recv(struct tevent_req *req,
 				     NTSTATUS *result);
 NTSTATUS dcerpc_lsa_GetUserName(struct dcerpc_binding_handle *h,
 				TALLOC_CTX *mem_ctx,
-				const char *_system_name /* [in] [unique,charset(UTF16)] */,
+				const char *_system_name /* [in] [charset(UTF16),unique] */,
 				struct lsa_String **_account_name /* [in,out] [ref] */,
 				struct lsa_String **_authority_name /* [in,out] [unique] */,
 				NTSTATUS *result);
@@ -992,7 +992,7 @@ struct tevent_req *dcerpc_lsa_SetInfoPolicy2_send(TALLOC_CTX *mem_ctx,
 						  struct dcerpc_binding_handle *h,
 						  struct policy_handle *_handle /* [in] [ref] */,
 						  enum lsa_PolicyInfo _level /* [in]  */,
-						  union lsa_PolicyInformation *_info /* [in] [ref,switch_is(level)] */);
+						  union lsa_PolicyInformation *_info /* [in] [switch_is(level),ref] */);
 NTSTATUS dcerpc_lsa_SetInfoPolicy2_recv(struct tevent_req *req,
 					TALLOC_CTX *mem_ctx,
 					NTSTATUS *result);
@@ -1000,7 +1000,7 @@ NTSTATUS dcerpc_lsa_SetInfoPolicy2(struct dcerpc_binding_handle *h,
 				   TALLOC_CTX *mem_ctx,
 				   struct policy_handle *_handle /* [in] [ref] */,
 				   enum lsa_PolicyInfo _level /* [in]  */,
-				   union lsa_PolicyInformation *_info /* [in] [ref,switch_is(level)] */,
+				   union lsa_PolicyInformation *_info /* [in] [switch_is(level),ref] */,
 				   NTSTATUS *result);
 
 struct tevent_req *dcerpc_lsa_QueryTrustedDomainInfoByName_r_send(TALLOC_CTX *mem_ctx,
@@ -1015,7 +1015,7 @@ struct tevent_req *dcerpc_lsa_QueryTrustedDomainInfoByName_send(TALLOC_CTX *mem_
 								struct policy_handle *_handle /* [in] [ref] */,
 								struct lsa_String *_trusted_domain /* [in] [ref] */,
 								enum lsa_TrustDomInfoEnum _level /* [in]  */,
-								union lsa_TrustedDomainInfo **_info /* [out] [ref,switch_is(level)] */);
+								union lsa_TrustedDomainInfo **_info /* [out] [switch_is(level),ref] */);
 NTSTATUS dcerpc_lsa_QueryTrustedDomainInfoByName_recv(struct tevent_req *req,
 						      TALLOC_CTX *mem_ctx,
 						      NTSTATUS *result);
@@ -1024,7 +1024,7 @@ NTSTATUS dcerpc_lsa_QueryTrustedDomainInfoByName(struct dcerpc_binding_handle *h
 						 struct policy_handle *_handle /* [in] [ref] */,
 						 struct lsa_String *_trusted_domain /* [in] [ref] */,
 						 enum lsa_TrustDomInfoEnum _level /* [in]  */,
-						 union lsa_TrustedDomainInfo **_info /* [out] [ref,switch_is(level)] */,
+						 union lsa_TrustedDomainInfo **_info /* [out] [switch_is(level),ref] */,
 						 NTSTATUS *result);
 
 struct tevent_req *dcerpc_lsa_SetTrustedDomainInfoByName_r_send(TALLOC_CTX *mem_ctx,
